@@ -47,6 +47,7 @@ namespace Oxygen
         _hideTitleBar( false ),
         _useDropShadows( true ),
         _useOxygenShadows( true ),
+        _closeFromMenuButton( false ),
         _useNarrowButtonSpacing( false ),
         _animationsEnabled( true ),
         _buttonAnimationsEnabled( true ),
@@ -127,6 +128,11 @@ namespace Oxygen
             OxygenConfig::USE_OXYGEN_SHADOWS,
             defaultConfiguration.useOxygenShadows() ) );
 
+        // close from menu button
+        setCloseFromMenuButton( group.readEntry(
+            OxygenConfig::CLOSE_FROM_MENU_BUTTON,
+            defaultConfiguration.closeFromMenuButton() ) );
+
         // buttonSpacing
         setUseNarrowButtonSpacing( group.readEntry(
             OxygenConfig::NARROW_BUTTON_SPACING,
@@ -207,6 +213,7 @@ namespace Oxygen
         if( hideTitleBar() != defaultConfiguration.hideTitleBar() ) group.writeEntry( OxygenConfig::HIDE_TITLEBAR, hideTitleBar() );
         if( useDropShadows() != defaultConfiguration.useDropShadows() ) group.writeEntry( OxygenConfig::USE_DROP_SHADOWS, useDropShadows() );
         if( useOxygenShadows() != defaultConfiguration.useOxygenShadows() ) group.writeEntry( OxygenConfig::USE_OXYGEN_SHADOWS, useOxygenShadows() );
+        if( closeFromMenuButton() != defaultConfiguration.closeFromMenuButton() ) group.writeEntry( OxygenConfig::CLOSE_FROM_MENU_BUTTON, closeFromMenuButton() );
         if( useNarrowButtonSpacing() != defaultConfiguration.useNarrowButtonSpacing() ) group.writeEntry( OxygenConfig::NARROW_BUTTON_SPACING, useNarrowButtonSpacing() );
 
         // animations
@@ -400,6 +407,7 @@ namespace Oxygen
             hideTitleBar() == other.hideTitleBar() &&
             useDropShadows() == other.useDropShadows() &&
             useOxygenShadows() == other.useOxygenShadows() &&
+            closeFromMenuButton() == other.closeFromMenuButton() &&
             useNarrowButtonSpacing() == other.useNarrowButtonSpacing() &&
             animationsEnabled() == other.animationsEnabled() &&
             buttonAnimationsEnabled() == other.buttonAnimationsEnabled() &&
