@@ -122,7 +122,17 @@ public:
         ActivateNextTabOp, // Move left in the group
         ActivatePreviousTabOp, // Move right in the group
         ToggleClientTiledStateOp, // put a floating client into tiling
-        TabDragOp
+        TabDragOp,
+
+        //BEGIN ABI stability stuff
+        // NOTICE for ABI stability
+        // TODO remove with mandatory version tagging fo 4.9.x or 4.10
+        /** @deprecated ABI compatibility only - don't use */
+        RemoveClientFromGroupOp = RemoveTabFromGroupOp, // Remove from group
+        CloseClientGroupOp = CloseTabGroupOp, // Close the group
+        MoveClientInGroupLeftOp = ActivateNextTabOp, // Move left in the group
+        MoveClientInGroupRightOp = ActivatePreviousTabOp // Move right in the group
+        //END ABI stability stuff
     };
     /**
      * Basic color types that should be recognized by all decoration styles.
@@ -212,7 +222,14 @@ public:
         // Tabbing
         AbilityTabbing = 4000, ///< The decoration supports tabbing
         // TODO colors for individual button types
-        ABILITY_DUMMY = 10000000
+        ABILITY_DUMMY = 10000000,
+
+        //BEGIN ABI stability stuff
+        // NOTICE for ABI stability
+        // TODO remove with mandatory version tagging fo 4.9.x or 4.10
+        /** @deprecated ABI compatibility only - don't use */
+        AbilityClientGrouping = AbilityTabbing
+        //END ABI stability stuff
     };
 
     enum Requirement { REQUIREMENT_DUMMY = 1000000 };
@@ -241,10 +258,39 @@ public:
     /**
      * Returns the mimeType used to drag and drop clientGroupItems
      */
+    //BEGIN ABI stability stuff
+    // NOTICE for ABI stability
+    // TODO remove with mandatory version tagging fo 4.9.x or 4.10
+    /** @deprecated ABI compatibility only - don't use */
+    static QString clientGroupItemDragMimeType() { return tabDragMimeType(); }
+    //END ABI stability stuff
     static QString tabDragMimeType();
 
 };
 
+//BEGIN ABI stability stuff
+// NOTICE for ABI stability
+// TODO remove with mandatory version tagging fo 4.9.x or 4.10
+/** @deprecated ABI compatibility only - don't use */
+class KWIN_EXPORT ClientGroupItem
+{
+public:
+    ClientGroupItem(QString t, QIcon i) {
+        title_ = t;
+        icon_ = i;
+    }
+    inline QIcon icon() const {
+        return icon_;
+    }
+    inline QString title() const {
+        return title_;
+    }
+private:
+    QString title_;
+    QIcon icon_;
+};
+//END ABI stability stuff
+
 class KDecorationProvides
     : public KDecorationDefines
 {
