@@ -362,13 +362,15 @@ void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quad
         WindowPaintData thumbData(thumb);
         thumbData.opacity = data.opacity;
 
-        QSizeF size = QSizeF(thumb->size());
+        const QRect visualThumbRect(thumb->expandedGeometry());
+
+        QSizeF size = QSizeF(visualThumbRect.size());
         size.scale(QSizeF(item->width(), item->height()), Qt::KeepAspectRatio);
-        if (size.width() > thumb->width() || size.height() > thumb->height()) {
-            size = QSizeF(thumb->size());
+        if (size.width() > visualThumbRect.width() || size.height() > visualThumbRect.height()) {
+            size = QSizeF(visualThumbRect.size());
         }
-        thumbData.xScale = size.width() / static_cast<qreal>(thumb->width());
-        thumbData.yScale = size.height() / static_cast<qreal>(thumb->height());
+        thumbData.xScale = size.width() / static_cast<qreal>(visualThumbRect.width());
+        thumbData.yScale = size.height() / static_cast<qreal>(visualThumbRect.height());
         // it can happen in the init/closing phase of the tabbox
         // that the corresponding QGraphicsScene is not available
         if (item->scene() == 0) {
@@ -407,7 +409,10 @@ void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quad
         const qreal x = point.x() + w->x() + (item->width() - size.width())/2;
         const qreal y = point.y() + w->y() + (item->height() - size.height()) / 2;
         thumbData.xTranslate = x - thumb->x();
+        // compensate shadow topleft padding
+        thumbData.xTranslate += (thumb->x()-visualThumbRect.x())*thumbData.xScale;
         thumbData.yTranslate = y - thumb->y();
+        thumbData.yTranslate += (thumb->y()-visualThumbRect.y())*thumbData.yScale;
         int thumbMask = PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS;
         if (thumbData.opacity == 1.0) {
             thumbMask |= PAINT_WINDOW_OPAQUE;
@@ -490,7 +495,7 @@ void Scene::Window::discardShape()
 
 // Find out the shape of the window using the XShape extension
 // or if shape is not set then simply it's the window geometry.
-QRegion Scene::Window::shape() const
+const QRegion &Scene::Window::shape() const
 {
     if (!shape_valid) {
         Client* c = dynamic_cast< Client* >(toplevel);
@@ -519,29 +524,28 @@ QRegion Scene::Window::shape() const
 
 QRegion Scene::Window::clientShape() const
 {
-    Client *c = dynamic_cast< Client* >(toplevel);
-    if (c && c->isShade())
-        return QRegion();
+    if (toplevel->isClient()) {
+        Client *c = static_cast< Client * > (toplevel);
+        if (c->isShade())
+            return QRegion();
+    }
 
+    // TODO: cache
     const QRegion r = shape() & QRect(toplevel->clientPos(), toplevel->clientSize());
     return r.isEmpty() ? QRegion() : r;
 }
 
 bool Scene::Window::isVisible() const
 {
-    if (dynamic_cast< Deleted* >(toplevel) != NULL)
+    if (toplevel->isDeleted())
         return false;
     if (!toplevel->isOnCurrentDesktop())
         return false;
     if (!toplevel->isOnCurrentActivity())
         return false;
-    if (Client* c = dynamic_cast< Client* >(toplevel))
-        return c->isShown(true);
+    if (toplevel->isClient())
+        return (static_cast< Client *>(toplevel))->isShown(true);
     return true; // Unmanaged is always visible
-    // TODO there may be transformations, so ignore this for now
-    return !toplevel->geometry()
-           .intersected(QRect(0, 0, displayWidth(), displayHeight()))
-           .isEmpty();
 }
 
 bool Scene::Window::isOpaque() const
@@ -557,13 +561,14 @@ bool Scene::Window::isPaintingEnabled() const
 void Scene::Window::resetPaintingEnabled()
 {
     disable_painting = 0;
-    if (dynamic_cast< Deleted* >(toplevel) != NULL)
+    if (toplevel->isDeleted())
         disable_painting |= PAINT_DISABLED_BY_DELETE;
     if (!toplevel->isOnCurrentDesktop())
         disable_painting |= PAINT_DISABLED_BY_DESKTOP;
     if (!toplevel->isOnCurrentActivity())
         disable_painting |= PAINT_DISABLED_BY_ACTIVITY;
-    if (Client* c = dynamic_cast< Client* >(toplevel)) {
+    if (toplevel->isClient()) {
+        Client *c = static_cast<Client*>(toplevel);
         if (c->isMinimized())
             disable_painting |= PAINT_DISABLED_BY_MINIMIZE;
         if (c->tabGroup() && c != c->tabGroup()->current())
@@ -612,7 +617,7 @@ WindowQuadList Scene::Window::buildQuads(bool force) const
     if (m_shadow) {
         ret << m_shadow->shadowQuads();
     }
-    effects->buildQuads(static_cast<Client*>(toplevel)->effectWindow(), ret);
+    effects->buildQuads(toplevel->effectWindow(), ret);
     cached_quad_list = new WindowQuadList(ret);
     return ret;
 }
