@@ -108,6 +108,12 @@ bool SceneOpenGL::db; // destination drawable is double-buffered
 #include "scene_opengl_glx.cpp"
 #endif
 
+void SceneOpenGL::idle()
+{
+    flushBuffer(m_lastMask, m_lastDamage);
+    Scene::idle();
+}
+
 bool SceneOpenGL::initFailed() const
 {
     return !init_ok;
@@ -254,6 +260,7 @@ void SceneOpenGL::windowGeometryShapeChanged(KWin::Toplevel* c)
 
 void SceneOpenGL::windowOpacityChanged(KWin::Toplevel* t)
 {
+    Q_UNUSED(t)
 #if 0 // not really needed, windows are painted on every repaint
     // and opacity is used when applying texture, not when
     // creating it
@@ -325,7 +332,6 @@ bool SceneOpenGL::Texture::load(const QImage& image, GLenum target)
 
 bool SceneOpenGL::Texture::load(const QPixmap& pixmap, GLenum target)
 {
-    Q_D(Texture);
     if (pixmap.isNull())
         return false;
 
@@ -445,22 +451,11 @@ QMatrix4x4 SceneOpenGL::Window::transformation(int mask, const WindowPaintData &
 // paint the window
 void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData data)
 {
-    // check if there is something to paint (e.g. don't paint if the window
-    // is only opaque and only PAINT_WINDOW_TRANSLUCENT is requested)
-    /* HACK: It seems this causes painting glitches, disable temporarily
-    bool opaque = isOpaque() && data.opacity == 1.0;
-    if (( mask & PAINT_WINDOW_OPAQUE ) ^ ( mask & PAINT_WINDOW_TRANSLUCENT ))
-        { // We are only painting either opaque OR translucent windows, not both
-        if ( mask & PAINT_WINDOW_OPAQUE && !opaque )
-            return; // Only painting opaque and window is translucent
-        if ( mask & PAINT_WINDOW_TRANSLUCENT && opaque )
-            return; // Only painting translucent and window is opaque
-        }*/
-
     if (region.isEmpty())
         return;
 
-    if (region != infiniteRegion() && !(mask & PAINT_WINDOW_TRANSFORMED)) {
+    bool hardwareClipping = region != infiniteRegion() && (mask & PAINT_WINDOW_TRANSFORMED);
+    if (region != infiniteRegion() && !hardwareClipping) {
         WindowQuadList quads;
         const QRegion filterRegion = region.translated(-x(), -y());
         // split all quads in bounding rect with the actual rects in the region
@@ -483,11 +478,16 @@ void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData
         data.quads = quads;
     }
 
-    if (!bindTexture())
+    if (!bindTexture()) {
         return;
+    }
+
+    if (hardwareClipping) {
+        glEnable(GL_SCISSOR_TEST);
+    }
 
     // Update the texture filter
-    if (options->glSmoothScale != 0 &&
+    if (options->glSmoothScale() != 0 &&
         (mask & (PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_TRANSFORMED)))
         filter = ImageFilterGood;
     else
@@ -523,7 +523,7 @@ void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData
 
     // shadow
     if (m_shadow) {
-        paintShadow(region, data);
+        paintShadow(region, data, hardwareClipping);
     }
     // decorations
     Client *client = dynamic_cast<Client*>(toplevel);
@@ -578,29 +578,34 @@ void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData
                 }
             }
 
-            paintDecoration(top, DecorationTop, region, topRect, data, topList, updateDeco);
-            paintDecoration(left, DecorationLeft, region, leftRect, data, leftList, updateDeco);
-            paintDecoration(right, DecorationRight, region, rightRect, data, rightList, updateDeco);
-            paintDecoration(bottom, DecorationBottom, region, bottomRect, data, bottomList, updateDeco);
+            paintDecoration(top, DecorationTop, region, topRect, data, topList, updateDeco, hardwareClipping);
+            paintDecoration(left, DecorationLeft, region, leftRect, data, leftList, updateDeco, hardwareClipping);
+            paintDecoration(right, DecorationRight, region, rightRect, data, rightList, updateDeco, hardwareClipping);
+            paintDecoration(bottom, DecorationBottom, region, bottomRect, data, bottomList, updateDeco, hardwareClipping);
         }
     }
 
     // paint the content
-    if (!(mask & PAINT_DECORATION_ONLY)) {
+    WindowQuadList contentQuads = data.quads.select(WindowQuadContents);
+    if (!contentQuads.empty()) {
         texture.bind();
         prepareStates(Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader);
-        renderQuads(mask, region, data.quads.select(WindowQuadContents), &texture);
+        renderQuads(mask, region, contentQuads, &texture, false, hardwareClipping);
         restoreStates(Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader);
         texture.unbind();
 #ifndef KWIN_HAVE_OPENGLES
         if (static_cast<SceneOpenGL*>(scene)->debug) {
             glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-            renderQuads(mask, region, data.quads.select(WindowQuadContents), &texture);
+            renderQuads(mask, region, contentQuads, &texture, false, hardwareClipping);
             glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
         }
 #endif
     }
 
+    if (hardwareClipping) {
+        glDisable(GL_SCISSOR_TEST);
+    }
+
     if (sceneShader) {
         ShaderManager::instance()->popShader();
         data.shader = NULL;
@@ -609,7 +614,9 @@ void SceneOpenGL::Window::performPaint(int mask, QRegion region, WindowPaintData
     }
 }
 
-void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType decorationType, const QRegion& region, const QRect& rect, const WindowPaintData& data, const WindowQuadList& quads, bool updateDeco)
+void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType decorationType,
+                                          const QRegion& region, const QRect& rect, const WindowPaintData& data,
+                                          const WindowQuadList& quads, bool updateDeco, bool hardwareClipping)
 {
     SceneOpenGL::Texture* decorationTexture;
     switch(decorationType) {
@@ -628,17 +635,16 @@ void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType
     default:
         return;
     }
-    if (decorationTexture->texture() != None && !updateDeco) {
-        // texture doesn't need updating, just bind it
-        decorationTexture->bind();
-    } else if (!decoration->isNull()) {
+    if (decoration->isNull()) {
+        return;
+    }
+    if (decorationTexture->isNull() || updateDeco) {
         bool success = decorationTexture->load(*decoration);
         if (!success) {
             kDebug(1212) << "Failed to bind decoartion";
             return;
         }
-    } else
-        return;
+    }
 
     // We have to update the texture although we do not paint anything.
     // This is especially needed if we draw the opaque part of the window
@@ -658,19 +664,19 @@ void SceneOpenGL::Window::paintDecoration(const QPixmap* decoration, TextureType
 
     prepareStates(decorationType, data.opacity * data.decoration_opacity, data.brightness, data.saturation, data.shader);
     makeDecorationArrays(quads, rect, decorationTexture);
-    GLVertexBuffer::streamingBuffer()->render(region, GL_TRIANGLES);
+    GLVertexBuffer::streamingBuffer()->render(region, GL_TRIANGLES, hardwareClipping);
     restoreStates(decorationType, data.opacity * data.decoration_opacity, data.brightness, data.saturation, data.shader);
     decorationTexture->unbind();
 #ifndef KWIN_HAVE_OPENGLES
     if (static_cast<SceneOpenGL*>(scene)->debug) {
         glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-        GLVertexBuffer::streamingBuffer()->render(region, GL_TRIANGLES);
+        GLVertexBuffer::streamingBuffer()->render(region, GL_TRIANGLES, hardwareClipping);
         glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     }
 #endif
 }
 
-void SceneOpenGL::Window::paintShadow(const QRegion &region, const WindowPaintData &data)
+void SceneOpenGL::Window::paintShadow(const QRegion &region, const WindowPaintData &data, bool hardwareClipping)
 {
     WindowQuadList quads = data.quads.select(WindowQuadShadowTopLeft);
     quads.append(data.quads.select(WindowQuadShadowTop));
@@ -694,13 +700,13 @@ void SceneOpenGL::Window::paintShadow(const QRegion &region, const WindowPaintDa
     texture->setWrapMode(GL_CLAMP_TO_EDGE);
     texture->bind();
     prepareStates(Shadow, data.opacity, data.brightness, data.saturation, data.shader, texture);
-    renderQuads(0, region, quads, texture, true);
+    renderQuads(0, region, quads, texture, true, hardwareClipping);
     restoreStates(Shadow, data.opacity, data.brightness, data.saturation, data.shader, texture);
     texture->unbind();
 #ifndef KWIN_HAVE_OPENGLES
     if (static_cast<SceneOpenGL*>(scene)->debug) {
         glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-        renderQuads(0, region, quads, texture);
+        renderQuads(0, region, quads, texture, true, hardwareClipping);
         glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     }
 #endif
@@ -765,7 +771,8 @@ void SceneOpenGL::Window::makeDecorationArrays(const WindowQuadList& quads, cons
     GLVertexBuffer::streamingBuffer()->setData(quads.count() * 6, 2, vertices.data(), texcoords.data());
 }
 
-void SceneOpenGL::Window::renderQuads(int, const QRegion& region, const WindowQuadList& quads, GLTexture *tex, bool normalized)
+void SceneOpenGL::Window::renderQuads(int, const QRegion& region, const WindowQuadList& quads,
+                                      GLTexture *tex, bool normalized, bool hardwareClipping)
 {
     if (quads.isEmpty())
         return;
@@ -785,7 +792,7 @@ void SceneOpenGL::Window::renderQuads(int, const QRegion& region, const WindowQu
 #endif
     quads.makeArrays(&vertices, &texcoords, size, tex->isYInverted());
     GLVertexBuffer::streamingBuffer()->setData(quads.count() * 6, 2, vertices, texcoords);
-    GLVertexBuffer::streamingBuffer()->render(region, GL_TRIANGLES);
+    GLVertexBuffer::streamingBuffer()->render(region, GL_TRIANGLES, hardwareClipping);
     delete[] vertices;
     delete[] texcoords;
 }
@@ -831,9 +838,6 @@ void SceneOpenGL::Window::prepareStates(TextureType type, double opacity, double
 void SceneOpenGL::Window::prepareShaderRenderStates(TextureType type, double opacity, double brightness, double saturation, GLShader* shader)
 {
     // setup blending of transparent windows
-#ifndef KWIN_HAVE_OPENGLES
-    glPushAttrib(GL_ENABLE_BIT);
-#endif
     bool opaque = isOpaque() && opacity == 1.0;
     bool alpha = toplevel->hasAlpha() || type != Content;
     if (type != Content)
@@ -1042,9 +1046,6 @@ void SceneOpenGL::Window::restoreShaderRenderStates(TextureType type, double opa
         glDisable(GL_BLEND);
     }
     ShaderManager::instance()->getBoundShader()->setUniform(GLShader::AlphaToOne, 0);
-#ifndef KWIN_HAVE_OPENGLES
-    glPopAttrib();  // ENABLE_BIT
-#endif
 }
 
 void SceneOpenGL::Window::restoreRenderStates(TextureType type, double opacity, double brightness, double saturation, GLTexture *tex)
@@ -1189,17 +1190,11 @@ void SceneOpenGL::EffectFrame::render(QRegion region, double opacity, double fra
         shader->setUniform(GLShader::AlphaToOne, 0);
     }
 
-#ifndef KWIN_HAVE_OPENGLES
-    glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT);
-#endif
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 #ifndef KWIN_HAVE_OPENGLES
     if (!shader)
         glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
-    // TODO: drop the push matrix
-    glPushMatrix();
 #endif
 
     // Render the actual frame
@@ -1472,10 +1467,6 @@ void SceneOpenGL::EffectFrame::render(QRegion region, double opacity, double fra
         ShaderManager::instance()->popShader();
     }
     glDisable(GL_BLEND);
-#ifndef KWIN_HAVE_OPENGLES
-    glPopMatrix();
-    glPopAttrib();
-#endif
 }
 
 void SceneOpenGL::EffectFrame::updateTexture()
