@@ -753,7 +753,7 @@ bool Launcher::eventFilter(QObject *object, QEvent *event)
     // deliver unhandled key presses from the search bar
     // (mainly arrow keys, enter) to the active view
     if ((object == d->contentSwitcher || object == d->searchBar) && event->type() == QEvent::KeyPress) {
-        // we want left/right to still nav the tabbar
+        // If the search text is empty, then we want left/right to still nav the tabbar.
         QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
         if (keyEvent->modifiers() == Qt::NoModifier &&
                 (keyEvent->key() == Qt::Key_Left ||
@@ -765,13 +765,21 @@ bool Launcher::eventFilter(QObject *object, QEvent *event)
                 return true;
             }
         }
+        // It works, using Key_Tab to enter or leave the flipScrollView versus
+        // other views, and arrow keys to move around inside. This decision tree
+        // is NOT able to figure out that Key_Up or Key_Down might "really want"
+        // to leave the FlipScrollView: If Key_Up is pressed at the top of a column,
+        // or Key_Down at the bottom, it does not move from the current
+        // highlighted item. It just sits there.
 
-        // Key_Up and Key_Down are meaningless on the tabbar and Search view.
-        // These keys indicate that the User wants to enter the currentWidget().
-        //
-        // The applicationView is a unique case, not reached via contentArea->currentWidget().
-        // If contentArea->currentWidget doesn't return an address,  we know that the
-        // applicationView is Highlighted/Selected. Set focus, and enter it.
+        if (d->searchView->isVisible()) {
+            if (keyEvent->key() == Qt::Key_Up ||
+                    keyEvent->key() == Qt::Key_Down ||
+                    keyEvent->key() == Qt::Key_Tab) {
+                d->searchView->setFocus();
+                QCoreApplication::sendEvent(d->searchView, event);
+            }
+        }
 
         if ((keyEvent->key() == Qt::Key_Up) || (keyEvent->key() == Qt::Key_Down)) {
             QAbstractItemView *activeView = qobject_cast<QAbstractItemView*>(d->contentArea->currentWidget());
@@ -783,19 +791,13 @@ bool Launcher::eventFilter(QObject *object, QEvent *event)
             }
             return true;
         }
-        // if the search view is visible, we are passing the events to it
-        if (d->searchView->isVisible()) {
-            if (!d->searchView->initializeSelection() ||
-                        keyEvent->key() == Qt::Key_Return ||
-                        keyEvent->key() == Qt::Key_Enter) {
-                QCoreApplication::sendEvent(d->searchView, event);
-            }
-            return true;
+        if (!d->searchView->initializeSelection() ||
+                keyEvent->key() == Qt::Key_Return ||
+                keyEvent->key() == Qt::Key_Enter) {
+             QCoreApplication::sendEvent(d->searchView, event);
+             return true;
         }
-
-        // getting for the active view, and passing the events to it
         QAbstractItemView *activeView = qobject_cast<QAbstractItemView*>(d->contentArea->currentWidget());
-
         if (activeView) {
             QCoreApplication::sendEvent(activeView, event);
             return true;
